%define anolis_release 1
%global majorversion 0
%global minorversion 3
%global microversion 74

%global apiversion   0.3
%global spaversion   0.2
%global ms_version   0.4.1

# Build conditions for various features
%bcond_without alsa
%bcond_without vulkan
%bcond_with media_session
%bcond_without pulse
%bcond_with jack
%bcond_with jackserver_plugin
%bcond_with libcamera_plugin
%bcond_with libusb
%bcond_without v4l2

Name:           pipewire
Version:        %{majorversion}.%{minorversion}.%{microversion}
Release:        %{anolis_release}%{?dist}
Summary:        Media Sharing Server
License:        MIT
URL:            https://%{name}.org/
Source0:        https://gitlab.freedesktop.org/%{name}/%{name}/-/archive/%{version}/%{name}-%{version}.tar.bz2


%if %{with media-session}
Source1:        https://gitlab.freedesktop.org/%{name}/media-session/-/archive/%{ms_version}/media-session-%{ms_version}.tar.gz
%endif

%if %{with media-session}
Patch1001:      0001-Build-media-session-from-local-tarbal.patch
%endif

BuildRequires:  gettext meson >= 0.49.0 gcc g++ doxygen python-docutils graphviz systemd
BuildRequires:  pkgconfig(libudev) pkgconfig(libmysofa) pkgconfig(dbus-1) pkgconfig(glib-2.0) >= 2.32
BuildRequires:  pkgconfig(gio-unix-2.0) >= 2.32 pkgconfig(fdk-aac) pkgconfig(bluez)
BuildRequires:  pkgconfig(gstreamer-1.0) >= 1.10.0 pkgconfig(gstreamer-base-1.0) >= 1.10.0
BuildRequires:  pkgconfig(gstreamer-plugins-base-1.0) >= 1.10.0 pkgconfig(gstreamer-net-1.0) >= 1.10.0 
BuildRequires:  pkgconfig(gstreamer-allocators-1.0) >= 1.10.0 pkgconfig(webrtc-audio-processing) >= 0.2
BuildRequires:  pkgconfig(ldacBT-enc) pkgconfig(ldacBT-abr) pkgconfig(libffado) pkgconfig(openssl) pkgconfig
%if %{with vulkan}
BuildRequires:  pkgconfig(vulkan)
%endif
BuildRequires:  systemd-devel alsa-lib-devel libv4l-devel sbc-devel libsndfile-devel
BuildRequires:  ncurses-devel pulseaudio-libs-devel avahi-devel readline-devel libcanberra-devel
%if %{with libusb}
BuildRequires:  libusb-devel
%endif

Requires(pre):  shadow-utils
Requires:       %{name}-libs = %{version}-%{release}
Requires:       systemd rtkit
# A virtual Provides so we can swap session managers
Requires:       %{name}-session-manager
# Prefer WirePlumber for session manager
Suggests:       wireplumber

%description
PipeWire is a multimedia server for Linux and other Unix like operating systems.

%package libs
Summary:        Libraries for PipeWire clients
License:        MIT
Recommends:     %{name} = %{version}-%{release}
Obsoletes:      %{name}-libpulse < %{version}

%description libs
This package contains the runtime libraries for any application that wishes to
interface with a PipeWire media server.

%package devel
Summary:        Headers and libraries for PipeWire client development
License:        MIT
Requires:       %{name}-libs = %{version}-%{release}

%description devel
Headers and libraries for developing applications that can communicate with a
PipeWire media server.

%package gstreamer
Summary:        GStreamer elements for PipeWire
License:        MIT
Recommends:     %{name} = %{version}-%{release}
Requires:       %{name}-libs = %{version}-%{release}

%description gstreamer
This package contains GStreamer elements to interface with a PipeWire media server.

%package utils
Summary:        PipeWire media server utilities
License:        MIT
Requires:       %{name} = %{version}-%{release}
Requires:       %{name}-libs = %{version}-%{release}

%description utils
This package contains command line utilities for the PipeWire media server.

%if %{with media_session}
%package media-session
Summary:        PipeWire Media Session Manager
License:        MIT
Conflicts:      %{name} < 0.3.30-5
Obsoletes:      %{name}-libpulse < %{version}
Recommends:     %{name} = %{version}-%{release}

# Virtual Provides to support swapping between PipeWire session manager implementations
Provides:       %{name}-session-manager
Conflicts:      %{name}-session-manager

%description media-session
This package contains the reference Media Session Manager for the PipeWire
media server.
%endif

%if %{with alsa}
%package alsa
Summary:        PipeWire media server ALSA support
License:        MIT
Obsoletes:      alsa-plugins-jack < 1.2.2
Obsoletes:      alsa-plugins-pulseaudio < 1.2.2
Recommends:     %{name} = %{version}-%{release}
Requires:       %{name}-libs = %{version}-%{release}
#zxt
Supplements:    %{name} = %{version}-%{release}

%description alsa
This package contains an ALSA plugin for the PipeWire media server.
%endif

%if %{with jack}
%package jack-audio-connection-kit
Summary:        PipeWire JACK implementation
License:        MIT
Recommends:     %{name} = %{version}-%{release}
Conflicts:      jack-audio-connection-kit jack-audio-connection-kit-dbus
# Fixed jack subpackages
Conflicts:      %{name}-libjack < 0.3.13-6
Conflicts:      %{name}-jack-audio-connection-kit < 0.3.13-6
# Replaces libjack subpackage
Obsoletes:      jack-audio-connection-kit < 1.9.16 %{name}-libjack < 0.3.19
Provides:       %{name}-libjack = %{version}-%{release}
Provides:       %{name}-libjack = %{version}-%{release}
Supplements:    %{name} = %{version}-%{release}

%description jack-audio-connection-kit
This package provides a JACK implementation based on PipeWire

%package jack-audio-connection-kit-devel
Summary:        Development files for %{name}-jack-audio-connection-kit
License:        MIT
Conflicts:      jack-audio-connection-kit-devel
Requires:       %{name}-jack-audio-connection-kit = %{version}-%{release}
Enhances:       %{name}-jack-audio-connection-kit

%description jack-audio-connection-kit-devel
This package provides development files for building JACK applications
using PipeWire's JACK library.
%endif

%if %{with jackserver_plugin}
%package plugin-jack
Summary:        PipeWire media server JACK support
License:        MIT
BuildRequires:  jack-audio-connection-kit-devel
Recommends:     %{name} = %{version}-%{release}
Requires:       %{name}-libs = %{version}-%{release}
Requires:       jack-audio-connection-kit

%description plugin-jack
This package contains the PipeWire spa plugin to connect to a JACK server.
%endif

%if %{with libcamera_plugin}
%package plugin-libcamera
Summary:        PipeWire media server libcamera support
License:        MIT
BuildRequires:  libcamera-devel
BuildRequires:  libdrm-devel
Recommends:     %{name} = %{version}-%{release}
Requires:       %{name}-libs = %{version}-%{release}
Requires:       libcamera libdrm

%description plugin-libcamera
This package contains the PipeWire spa plugin to access cameras through libcamera.
%endif

%if %{with pulse}
%package pulseaudio
Summary:        PipeWire PulseAudio implementation
License:        MIT
Recommends:     %{name} = %{version}-%{release}
Requires:       %{name}-libs = %{version}-%{release}
BuildRequires:  pulseaudio-libs
Conflicts:      pulseaudio
# Fixed pulseaudio subpackages
Conflicts:      %{name}-libpulse < 0.3.13-6
Conflicts:      %{name}-pulseaudio < 0.3.13-6
Obsoletes:      pulseaudio < 14.2
Obsoletes:      pulseaudio-esound-compat < 14.2
Obsoletes:      pulseaudio-module-bluetooth < 14.2
Obsoletes:      pulseaudio-module-gconf < 14.2
Obsoletes:      pulseaudio-module-gsettings < 14.2
Obsoletes:      pulseaudio-module-jack < 14.2
Obsoletes:      pulseaudio-module-lirc < 14.2
Obsoletes:      pulseaudio-module-x11 < 14.2
Obsoletes:      pulseaudio-module-zeroconf < 14.2
Obsoletes:      pulseaudio-qpaeq < 14.2
Supplements:    %{name} = %{version}-%{release}

# Virtual Provides to support swapping between PipeWire-PA and PA
Conflicts:      pulseaudio-daemon
Provides:       pulseaudio-daemon
Provides:       pulseaudio-module-bluetooth
Provides:       pulseaudio-module-jack

%description pulseaudio
This package provides a PulseAudio implementation based on PipeWire
%endif

%if %{with v4l2}
%package v4l2
Summary:        PipeWire media server v4l2 LD_PRELOAD support
License:        MIT
Recommends:     %{name} = %{version}-%{release}
Requires:       %{name}-libs = %{version}-%{release}

%description v4l2
This package contains an LD_PRELOAD library that redirects v4l2 applications to
PipeWire.
%endif

%package doc
Summary:        PipeWire media server documentation
License:        MIT

%description doc
This package contains documentation for the PipeWire media server.

%prep
%autosetup -n %{name}-%{version} -p1

%if %{with media-session}
mkdir subprojects/packagefiles
install -m 0644 -p %{SOURCE1} subprojects/packagefiles/
%endif

%build
%meson \
    -D audiotestsrc=disabled -D videotestsrc=disabled				\
    -D bluez5-codec-lc3plus=disabled                                            \
    -D docs=enabled -D man=enabled -D gstreamer=enabled -D systemd=enabled	\
    -D gstreamer-device-provider=disabled -D sdl2=disabled 			\
    -D lv2=disabled                                          			\
    -D volume=disabled -D bluez5-codec-aptx=disabled -D roc=disabled 		\
    %{!?with_media_session:-D session-managers=[]} 				\
    %{!?with_jack:-D %{name}-jack=disabled} 					\
    %{!?with_jackserver_plugin:-D jack=disabled} 				\
    %{!?with_libcamera_plugin:-D libcamera=disabled} 				\
    %{?with_jack:-D jack-devel=true} 						\
    %{!?with_alsa:-D %{name}-alsa=disabled}					\
    %{?with_vulkan:-D vulkan=enabled}						\
    %{!?with_libusb:-D libusb=disabled}
%meson_build

%install
%meson_install

%if %{with jack}
mkdir -p %{buildroot}%{_sysconfdir}/ld.so.conf.d/
echo %{_libdir}/%{name}-%{apiversion}/jack/ > %{buildroot}%{_sysconfdir}/ld.so.conf.d/%{name}-jack-%{_arch}.conf
%else
rm %{buildroot}%{_datadir}/%{name}/jack.conf

%if %{with media_session}
rm %{buildroot}%{_datadir}/%{name}/media-session.d/with-jack
%endif

%endif

%if %{with alsa}
mkdir -p %{buildroot}%{_sysconfdir}/alsa/conf.d/
install -m 0644 -p %{buildroot}%{_datadir}/alsa/alsa.conf.d/50-%{name}.conf \
        %{buildroot}%{_sysconfdir}/alsa/conf.d/50-%{name}.conf
install -m 0644 -p %{buildroot}%{_datadir}/alsa/alsa.conf.d/99-%{name}-default.conf \
        %{buildroot}%{_sysconfdir}/alsa/conf.d/99-%{name}-default.conf

%if %{with media_session}
touch %{buildroot}%{_datadir}/%{name}/media-session.d/with-alsa
%endif

%endif

%if ! %{with pulse}
# If the PulseAudio replacement isn't being offered, delete the files
rm %{buildroot}%{_bindir}/%{name}-pulse
rm %{buildroot}%{_userunitdir}/%{name}-pulse.*
rm %{buildroot}%{_datadir}/%{name}/%{name}-pulse.conf

%if %{with media_session}
rm %{buildroot}%{_datadir}/%{name}/media-session.d/with-pulseaudio
%endif

%endif

%find_lang %{name}
%if %{with media_session}
%find_lang media-session
%endif

mkdir -p %{buildroot}%{_prefix}/lib/udev/rules.d
mv -fv %{buildroot}/lib/udev/rules.d/90-%{name}-alsa.rules %{buildroot}%{_prefix}/lib/udev/rules.d


%check
%meson_test || TESTS_ERROR=$?
if [ "${TESTS_ERROR}" != "" ]; then
echo "test failed"
%{!?tests_nonfatal:exit $TESTS_ERROR}
fi

%pre
getent group %{name} >/dev/null || groupadd -r %{name}
getent passwd %{name} >/dev/null || \
    useradd -r -g %{name} -d %{_localstatedir}/run/%{name} -s /sbin/nologin -c "PipeWire System Daemon" %{name}
exit 0

%post
%systemd_user_post %{name}.service
%systemd_user_post %{name}.socket

%triggerun -- %{name} < 0.3.6-2
# The %%post scriptlet above only does anything on initial package installation.
systemctl --no-reload preset --global %{name}.socket >/dev/null 2>&1 || :

%if %{with pulse}
%post pulseaudio
%systemd_user_post %{name}-pulse.service
%systemd_user_post %{name}-pulse.socket
%endif

%if %{with media_session}
%post media-session
%systemd_user_post %{name}-media-session.service
%endif

%files
%license LICENSE COPYING
%{_userunitdir}/%{name}.*
%{_userunitdir}/filter-chain.*
%{_bindir}/%{name}
%{_bindir}/%{name}-aes67
%{_bindir}/%{name}-avb
%{_mandir}/man1/%{name}.1*
%dir %{_datadir}/%{name}/
%{_datadir}/%{name}/%{name}-avb.conf
%{_datadir}/%{name}/%{name}-aes67.conf
%{_datadir}/%{name}/%{name}.conf
%{_datadir}/%{name}/%{name}.conf.avail/10-rates.conf            
%{_datadir}/%{name}/%{name}.conf.avail/20-upmix.conf
%{_datadir}/%{name}/filter-chain.conf
%{_datadir}/%{name}/filter-chain/*.conf
%{_datadir}/%{name}/minimal.conf
%{_mandir}/man5/%{name}.conf.5*
%config(noreplace) %{_sysconfdir}/security/limits.d/*.conf

%files libs -f %{name}.lang
%license LICENSE COPYING
%{_libdir}/lib%{name}-%{apiversion}.so.*
%{_libdir}/%{name}-%{apiversion}/lib%{name}-*.so
%dir %{_datadir}/alsa-card-profile/
%dir %{_datadir}/alsa-card-profile/mixer/
%{_datadir}/alsa-card-profile/mixer/paths/
%{_datadir}/alsa-card-profile/mixer/profile-sets/
%dir %{_datadir}/spa-0.2/
%{_datadir}/spa-0.2/bluez5/bluez-hardware.conf
%{_prefix}/lib/udev/rules.d/90-%{name}-alsa.rules
%dir %{_libdir}/spa-%{spaversion}
%{_libdir}/spa-%{spaversion}/aec/
%{_libdir}/spa-%{spaversion}/alsa/
%{_libdir}/spa-%{spaversion}/audioconvert/
%{_libdir}/spa-%{spaversion}/audiomixer/
%{_libdir}/spa-%{spaversion}/avb/
%{_libdir}/spa-%{spaversion}/bluez5/
%{_libdir}/spa-%{spaversion}/control/
%{_libdir}/spa-%{spaversion}/support/
%{_libdir}/spa-%{spaversion}/v4l2/
%{_libdir}/spa-%{spaversion}/videoconvert/
%if %{with vulkan}
%{_libdir}/spa-%{spaversion}/vulkan/
%endif
%{_datadir}/%{name}/client.conf
%{_datadir}/%{name}/client.conf.avail/20-upmix.conf
%{_datadir}/%{name}/client-rt.conf
%{_datadir}/%{name}/client-rt.conf.avail/20-upmix.conf

%files devel
%{_includedir}/spa-%{spaversion}/
%{_includedir}/%{name}-%{apiversion}/
%{_libdir}/lib%{name}-%{apiversion}.so
%{_libdir}/pkgconfig/lib%{name}-%{apiversion}.pc
%{_libdir}/pkgconfig/libspa-%{spaversion}.pc

%files gstreamer
%{_libdir}/gstreamer-1.0/libgst%{name}.*

%if %{with media_session}
%files media-session -f media-session.lang
%{_bindir}/%{name}-media-session
%{_userunitdir}/%{name}-media-session.service
%dir %{_datadir}/%{name}/media-session.d/
%{_datadir}/%{name}/media-session.d/alsa-monitor.conf
%{_datadir}/%{name}/media-session.d/bluez-monitor.conf
%{_datadir}/%{name}/media-session.d/media-session.conf
%{_datadir}/%{name}/media-session.d/v4l2-monitor.conf

%if %{with alsa}
%{_datadir}/%{name}/media-session.d/with-alsa
%endif
%if %{with jack}
%{_datadir}/%{name}/media-session.d/with-jack
%endif
%if %{with pulse}
%{_datadir}/%{name}/media-session.d/with-pulseaudio
%endif

%endif

%files utils
%{_bindir}/pw-cat
%{_bindir}/pw-cli
%{_bindir}/pw-config
%{_bindir}/pw-dot
%{_bindir}/pw-dump
%{_bindir}/pw-dsdplay
%{_bindir}/pw-encplay
%{_bindir}/pw-link
%{_bindir}/pw-loopback
%{_bindir}/pw-mon
%{_bindir}/pw-metadata
%{_bindir}/pw-mididump
%{_bindir}/pw-midiplay
%{_bindir}/pw-midirecord
%{_bindir}/pw-play
%{_bindir}/pw-profiler
%{_bindir}/pw-record
%{_bindir}/pw-reserve
%{_bindir}/pw-top
%{_bindir}/spa-acp-tool
%{_bindir}/spa-inspect
%{_bindir}/spa-json-dump
%{_bindir}/spa-monitor
%{_bindir}/spa-resample
%{_mandir}/man1/pw-cli.1*
%{_mandir}/man1/pw-cat.1*
%{_mandir}/man1/pw-config.1*
%{_mandir}/man1/pw-dot.1*
%{_mandir}/man1/pw-link.1*
%{_mandir}/man1/pw-mon.1*
%{_mandir}/man1/pw-metadata.1*
%{_mandir}/man1/pw-mididump.1*
%{_mandir}/man1/pw-profiler.1*
%{_mandir}/man1/pw-top.1*

%if %{with alsa}
%files alsa
%{_datadir}/alsa/alsa.conf.d/50-%{name}.conf
%{_datadir}/alsa/alsa.conf.d/99-%{name}-default.conf
%{_libdir}/alsa-lib/libasound_module_pcm_%{name}.so
%{_libdir}/alsa-lib/libasound_module_ctl_%{name}.so
%config(noreplace) %{_sysconfdir}/alsa/conf.d/50-%{name}.conf
%config(noreplace) %{_sysconfdir}/alsa/conf.d/99-%{name}-default.conf
%endif

%if %{with jack}
%files jack-audio-connection-kit
%{_bindir}/pw-jack
%{_datadir}/%{name}/jack.conf
%{_libdir}/%{name}-%{apiversion}/jack/libjack.so.*
%{_libdir}/%{name}-%{apiversion}/jack/libjacknet.so.*
%{_libdir}/%{name}-%{apiversion}/jack/libjackserver.so.*
%{_mandir}/man1/pw-jack.1*
%{_sysconfdir}/ld.so.conf.d/%{name}-jack-%{_arch}.conf

%files jack-audio-connection-kit-devel
%{_includedir}/jack/
%{_libdir}/%{name}-%{apiversion}/jack/libjack.so
%{_libdir}/%{name}-%{apiversion}/jack/libjacknet.so
%{_libdir}/%{name}-%{apiversion}/jack/libjackserver.so
%{_libdir}/pkgconfig/jack.pc
%endif

%if %{with jackserver_plugin}
%files plugin-jack
%{_libdir}/spa-%{spaversion}/jack/
%endif

%if %{with libcamera_plugin}
%files plugin-libcamera
%{_libdir}/spa-%{spaversion}/libcamera/
%endif

%if %{with pulse}
%files pulseaudio
%{_bindir}/%{name}-pulse
%{_datadir}/%{name}/%{name}-pulse.conf
%{_datadir}/%{name}/%{name}-pulse.conf.avail/20-upmix.conf
%{_mandir}/man1/%{name}-pulse.1*
%{_userunitdir}/%{name}-pulse.*
%endif

%if %{with v4l2}
%files v4l2
%{_bindir}/pw-v4l2
%{_libdir}/%{name}-%{apiversion}/v4l2/libpw-v4l2.so
%endif

%files doc
%{_datadir}/doc/%{name}/html
%doc README.md

%changelog
* Wed Jul 12 2023 Funda Wang <fundawang@yeah.net> - 0.3.74-1
- New version 0.3.74

* Fri Apr 21 2023 Funda Wang <fundawang@yeah.net> - 0.3.70-1
- New version 0.3.70

* Sat Apr 15 2023 Funda Wang <fundawang@yeah.net> - 0.3.69-1
- New version 0.3.69

* Fri Apr 14 2023 Jing Zhang <renyu.zj@linux.alibaba.com> - 0.3.68-2
- Refact spec

* Thu Apr 6 2023 Funda Wang <fundawang@yeah.net> - 0.3.68-1
- New version 0.3.68

* Fri Mar 10 2023 Funda Wang <fundawang@yeah.net> - 0.3.67-1
- New version 0.3.67

* Thu Feb 16 2023 Funda Wang <fundawang@yeah.net> - 0.3.66-1
- New verison 0.3.66
- Disable supports of jack, libcamera as we don't have them yet

* Tue Jul 26 2022 Chunmei Xu <xuchunmei@linux.alibaba.com> - 0.3.49-2
- disable lv2 and raop

* Thu Apr 21 2022 zhang xianting <zhangxianting@uniontech.com> - 0.3.49-1
- Initial build for Anolis V23
